# Copyright 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt
from __future__ import absolute_import
import time
from logging import DEBUG, INFO, WARNING, ERROR, CRITICAL
from Queue import Queue, Empty
from netq_apps.lib.integrations import Integration
from netq_apps.lib.events import EventService
from netq_lib.common import configuration
_PROC_NAME = 'netq-eventd'

from netq_apps.lib.event_defs import (
        MESSAGE, SEVERITY,
        DATA, TYPE,
        MSGS, NAME,
        DIFF, TIME
)


from netq_events.lib.event_database import (
    EventDatabase,
    EventObject,
    NANOSECONDS,
    BATCH_SIZE
)

class InfluxDb(Integration):

    def __init__(self, proc_name, logger, integ):
        self.db = EventDatabase(logger=logger)
        self.event_queue = Queue()
        self._logger = logger
        super(self.__class__, self).__init__(proc_name, logger, integ)

    def verify_conf(self, conf):
        return True

    def populate_request(self, conf):
        try:
            self.conf['event_db'] = EventDatabase(logger=self._logger)
            self.conf['log_severity'] = {
                INFO: 'INFO', DEBUG: 'DEBUG', WARNING: 'WARNING',
                ERROR: 'ERROR', CRITICAL: 'CRITICAL'}
        except:
            self.log_integration(WARNING, 'Failed to initialize Event database')
        self.conf['event_db'] = None

    def post(self, msg):
        m_type = msg.get('type', 'unknown')

        if not self.db:
            self.log_integration(INFO, 'Event database not initialized!!! Attempting to initialize')
            self.db = EventDatabase(logger=self._logger)

        if msg.get(DIFF):
            event_object = EventObject(
                timestamp=int(msg.get(TIME, time.time())*NANOSECONDS),
                eventType=m_type,
                severity=self.conf['log_severity'][msg.get(SEVERITY, INFO)],
                diff=msg.get(DIFF, ''),
                message=msg.get(MESSAGE))

            if not self.db.is_connected():
                if not self.db.initialize_db():
                    self.log_integration(INFO, 'Adding event to Influx database queue...')
                    self.event_queue.put(event_object)
                    return
            self.db.save(event_object)
            self.db.commit()


class EventdService(EventService):

    def empty_handler(self):
        self.global_msg_flush()
        self.empty_conn_handler()
        influxdb = self.integrations.get('influxdb', None)
        if not influxdb:
            self._logger.debug("Failed to find influxdb object...")
            return

        if not influxdb.db:
            self._logger.debug("Influx database not iniialized. Attempt to reinit... ")
            influxdb.db = EventDatabase(logger=self._logger)

        if influxdb.db and not influxdb.db.is_connected():
            if not influxdb.db.initialize_db():
                self._logger.debug("Skip pushing info. Influx database connection failed...")
                return

        count = 0
        while not influxdb.event_queue.empty():
            event = influxdb.event_queue.get_nowait()
            influxdb.db.save(event)
            count = count + 1
            if count > BATCH_SIZE:
                break

        if count:
            self._logger.debug("Dequeue {} events from the queue".format(count))
            influxdb.db.commit()


def main():
    conf_dict = configuration.get_config(_PROC_NAME, True)
    conf_dict['event-integrations'] = [{'type': 'influxdb', 'name': 'influxdb'}]
    conf_dict['event-filters'] = [{'output': ['influxdb'], 'name': 'default',
                                   'rule': None}]
    integ_map = {'influxdb': InfluxDb}
    event_inst = EventdService(conf_dict, _PROC_NAME, integ_map,
                              'event-filters', 'event-integrations')
    return event_inst.run()
