# Copyright 2016, 2017 Cumulus Networks, Inc.  All Rights reserved.
#
# This software is subject to the Cumulus Networks End User License Agreement
# available at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

"""ROM models
"""
import inspect
from influxdb import InfluxDBClient
from netq_lib.common import utils

class EventDatabaseException(Exception):
    pass

class EventFieldException(Exception):
    pass

class EventObjectException(Exception):
    pass


EventType = ['BGP', 'LNV', 'Link', 'CLAG', 'NTP', 'Services', 'Port', 'OS',
             'License', 'PSU', 'Fan', 'Temp', 'NetQ Agent']
EventSeverity = ['CRITICAL', 'WARNING', 'INFO', 'DEBUG']

NANOSECONDS = 1000000000
BATCH_SIZE = 1000
_PROC_NAME = 'event_database'


class EventObject(object):
    timestamp = int
    eventType = str
    severity = str
    message = str

    def __init__(self, **kwargs):
        for key, value in self.__class__.__dict__.items():
            if not isinstance(value, type):
                continue

            if key in kwargs:
                if not isinstance(kwargs[key], value):
                    errorString = "{} should be {}".format(key, value)
                    raise EventFieldException(errorString)
                self.__dict__[key] = value(kwargs[key])
        return

    def json_output(self):
        value = {}
        value['timestamp'] = getattr(self, 'timestamp')
        value['eventType'] = getattr(self, 'eventType')
        value['severity'] = getattr(self, 'severity')
        value['message'] = getattr(self, 'message')
        return "{}".format(value)

    def save(self, database):
        try:
            database.save(self)
        except Exception as e:
            raise EventObjectException("database save operation failed")

    def tags(self):
        value = {}
        value['severity'] = getattr(self, 'severity')
        value['eventType'] = getattr(self, 'eventType')
        return value

    def fields(self):
        value = {}
        value['message'] = getattr(self, 'message')
        return value

    def time(self):
        return getattr(self, 'timestamp')


class EventDatabase(object):
    """Class to interface with Event database"""
    __DB_NAME = 'netqEventDatabase'
    __MEASUREMENT = 'netqEvent'
    __PRECISION = 'ms'

    def initialize_db(self):
        try:
            self.client = InfluxDBClient(database=self.db_name)
            self.client.create_database(self.db_name)
        except Exception as e:
            self.client = None
            return False

        return True


    def __init__(self, **kwargs):
        self.db_name = kwargs.get('db_name', self.__DB_NAME)
        self.measurement = kwargs.get('measurement', self.__MEASUREMENT)
        self.precision = kwargs.get('precision', None)
        self._logger = kwargs.get('logger', None)
        if not self._logger:
            self._logger = utils.get_logger(_PROC_NAME)

        if not self.initialize_db():
            self._logger.debug("Failed to initialize influx database")

        self.dataPoints = []
        return


    def is_connected(self):
        if not self.client:
            return False

        try:
            _ = self.client.get_list_database()
        except Exception as e:
            self.client = None
            return False
        return True


    def save(self, event, measurement=None):
        dataPoint = {}
        if measurement:
            dataPoint["measurement"] = measurement
        else:
            dataPoint["measurement"] = self.measurement
        dataPoint["tags"] = event.tags()
        dataPoint["fields"] = event.fields()
        dataPoint["time"] = event.time()
        self.dataPoints.append(dataPoint)


    def commit(self):
        if not self.dataPoints:
            self._logger.debug("Nothing to commit...")
            return

        if not self.is_connected():
            if not self.initialize_db():
                self._logger.debug("Commit skipped: Database not initialized")
                return

        try:
            if not self.precision:
                self.client.write_points(self.dataPoints)
            else:
                self.client.write_points(self.dataPoints, time_precision=self.precision)
        except Exception as e:
            raise EventDatabaseException("Failed to commit data points to influxdb")
        self.dataPoints = []
        return


    def query(self, queryString):
        if not self.is_connected():
            if not self.initialize_db():
                self._logger.debug("Database not initialized")
                yield None

        try:
            result = self.client.query(queryString)
            for datapoint in result.get_points():
                yield datapoint
        except Exception as e:
            yield None

    def raw_count(self, queryString):
        value = 0

        if not self.is_connected():
            if not self.initialize_db():
                self._logger.debug("Database not initialized")
                return value

        try:
            result = self.client.query(queryString)
            for datapoint in result.get_points():
                if 'count_message' in datapoint:
                    value = datapoint['count_message']
        except Exception as e:
            pass

        return value


    def event_count(self, startTime, endTime, interval, filters={}):
        eventTypeFilter = filters.get('eventType', None)
        severityFilter = filters.get('severity', None)

        data = {}
        if not self.is_connected():
            if not self.initialize_db():
                data['startTimes'] = []
                data['eventCounts'] = {}
                self._logger.debug("Database not initialized")
                return data

        startTimeList = [ ts for ts in range(startTime, endTime, interval) ]
        data['startTimes'] = startTimeList

        eventCounts = {}
        for eventType in EventType:
            # If filter is defined, include only the event_types specified 
            # in the filter (it's an include filter)
            if eventTypeFilter and eventType not in eventTypeFilter:
                continue

            counterList = []
            for ts in startTimeList:
                counters = {}
                total = 0

                for severity in EventSeverity:
                    if severityFilter and severity not in severityFilter:
                        continue

                    queryString = "select count(*) from {} where eventType = \'{}\' and\
                            severity = \'{}\' and time > {} and time <= {}".format(
                                self.__MEASUREMENT, eventType, severity, ts, ts + interval)

                    counters[severity] = self.raw_count(queryString)
                    total = total + counters[severity]
                counters['total'] = total
                counterList.append(counters)
            eventCounts[eventType] = counterList
        data['eventCounts'] = eventCounts
        return data


    def event_messages(self, startTime, endTime, filters={}):
        eventTypeFilter = filters.get('eventType', None)
        severityFilter = filters.get('severity', None)

        event_messages_list = []

        if not self.is_connected():
            if not self.initialize_db():
                self._logger.debug("Database not initialized")
                return event_messages_list

        for eventType in EventType:
            # If filter is defined, include only the event_types specified 
            # in the filter (it's an include filter)
            if eventTypeFilter and eventType not in eventTypeFilter:
                continue
            for severity in EventSeverity:
                if severityFilter and severity not in severityFilter:
                    continue
                queryString = "select * from {} where eventType = \'{}\' and\
                                severity = \'{}\' and time > {} and time <= {}".format(
                                self.__MEASUREMENT,  eventType, severity, startTime, endTime)
                for dataPoint in self.query(queryString):
                    event_messages_list.append(dataPoint)
        return event_messages_list


if __name__ == '__main__':
    db = EventDatabase()
    e = EventObject(timestamp=1001, eventType='BGP', severity='CRITICAL', message='Session down')
    db.save(e)
    e = EventObject(timestamp=1002, eventType='BGP', severity='WARNING', message='Session down')
    db.save(e)
    db.commit()
    print db.event_count(1, 2000, 1000)
    print db.event_count(1, 2000, 1000, {'severity': 'WARNING'})
    print db.event_messages(1, 2000, {'eventType': ['BGP'], 'severity': ['CRITICAL']})
